<?php
/**
 * Plugin Name: Tickera - Attendees Gravatars
 * Plugin URI: https://tickera.com/
 * Description: This add-on adds event attendees gravatars to a page. Just paste shortcode [tc_attendees_gravatars] in the event's content. Optionally add id (event's id) if needed like this [tc_attendees_gravatars id="123"]
 * Version: 2.0
 * Author: Tickera
 * Author URI: https://tickera.com/
 * Developer: Tickera
 * Developer URI: https://tickera.com/
 */

if ( !defined( 'ABSPATH' ) )
    exit; // Exit if accessed directly

if ( !class_exists( 'TC_Attendees_Gravatars' ) ) {

    class TC_Attendees_Gravatars {

        var $version = '2.0';
        var $title = 'Tickera - Attendees Gravatars';
        var $name = 'custom-attendees-gravatars';
        var $dir_name = 'custom-attendees-gravatars';
        var $location = 'plugins';
        var $plugin_dir = '';
        var $plugin_url = '';
        var $valid_order_statuses = [ 'order_paid', 'wc-completed' ];

        function __construct() {
            self::init_vars();
            add_action( 'activated_plugin', array( $this, 'load_this_plugin_the_very_last' ) );
            add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
            add_shortcode( 'tc_attendees_gravatars', array( $this, 'render_shortcode' ) );
        }

        function init_vars() {

            $general_settings = get_option('tc_general_setting', false);
            $woo_allow_cash_on_delivery_processing = isset( $general_settings['tc_woo_allow_cash_on_delivery_processing'] ) ? $general_settings['tc_woo_allow_cash_on_delivery_processing'] : 'no';
            if ( 'yes' == $woo_allow_cash_on_delivery_processing ) $this->valid_order_statuses[] = 'wc-processing';
            $this->valid_order_statuses = apply_filters( 'tc_attendees_gravatars_valid_order_statuses', $this->valid_order_statuses );

            if ( defined( 'WP_PLUGIN_URL' ) && defined( 'WP_PLUGIN_DIR' ) && file_exists( WP_PLUGIN_DIR . '/' . $this->dir_name . '/' . basename( __FILE__ ) ) ) {
                $this->location = 'subfolder-plugins';
                $this->plugin_dir = WP_PLUGIN_DIR . '/' . $this->dir_name . '/';
                $this->plugin_url = plugins_url( '/', __FILE__ );

            } elseif ( defined( 'WP_PLUGIN_URL' ) && defined( 'WP_PLUGIN_DIR' ) && file_exists( WP_PLUGIN_DIR . '/' . basename( __FILE__ ) ) ) {
                $this->location = 'plugins';
                $this->plugin_dir = WP_PLUGIN_DIR . '/';
                $this->plugin_url = plugins_url( '/', __FILE__ );

            } elseif ( is_multisite() && defined( 'WPMU_PLUGIN_URL' ) && defined( 'WPMU_PLUGIN_DIR' ) && file_exists( WPMU_PLUGIN_DIR . '/' . basename( __FILE__ ) ) ) {
                $this->location = 'mu-plugins';
                $this->plugin_dir = WPMU_PLUGIN_DIR;
                $this->plugin_url = WPMU_PLUGIN_URL;

            } else {
                wp_die( sprintf( __( 'There was an issue determining where %s is installed. Please reinstall it.', 'tc' ), $this->title ) );
            }
        }

        /**
         * Modify plugin load sequence.
         * Make sure to load dependencies first.
         */
        function load_this_plugin_the_very_last() {
            $path = $this->dir_name . '/' . basename( __FILE__ );
            if ( $plugins = get_option( 'active_plugins' ) ) {
                if ( $key = array_search( $path, $plugins ) ) {
                    unset( $plugins[ $key ] );
                    $plugins[] = $path;
                    update_option( 'active_plugins', array_values( $plugins ) );
                }
            }
        }

        function enqueue_scripts() {
            wp_enqueue_style( 'tc_attendees_gravatars', plugin_dir_url( __FILE__ ) . 'assets/css/gravatars.css', array(), '1.0' );
        }

        function render_shortcode( $atts ) {

            ob_start();
            global $post;

            extract( shortcode_atts( [
                'id' => false,
                'size' => 60,
                'default' => false,
                'show_attendee_name' => true,
                'use_buyer_data_if_attendees_not_available' => false,
                'wrapper_class' => 'tc_attendees_gravatars_wrap',
                'holder_class' => 'tc_attendee_gravatar_holder'
            ], $atts ) );

            $id = ( empty( $id ) || !$id ) ? $post->ID : $id;

            $args = array(
                'post_type' => 'tc_tickets_instances',
                'post_status' => 'publish',
                'meta_key' => 'event_id',
                'meta_value' => $id,
                'fields' => 'ids',
                'posts_per_page' => '-1'
            );

            if ( ! $use_buyer_data_if_attendees_not_available ) {
                $args[ 'meta_query' ] = [
                    'relation' => 'AND',
                    [ 'key' => 'first_name', 'compare' => 'EXISTS' ],
                    [ 'key' => 'last_name', 'compare' => 'EXISTS' ]
                ];
            }

            $attendee_ids = get_posts( $args );

            // Format data
            $attendee_data = [];
            foreach ( $attendee_ids as $attendee_id ) {

                $ticket = new TC_Ticket( $attendee_id );
                $order_id = wp_get_post_parent_id( $attendee_id );
                $order = new TC_Order( $order_id );

                if ( in_array( $order->details->post_status, $this->valid_order_statuses ) ) {
                    $first_name = sanitize_text_field( $ticket->details->first_name );
                    $last_name = sanitize_text_field( $ticket->details->last_name );
                    $attendee_name = $first_name . ' ' . $last_name;
                    $owner_email = sanitize_email( $ticket->details->owner_email );

                    if ( $use_buyer_data_if_attendees_not_available && !trim( $attendee_name ) ) {

                        if ( 'tc_orders' == $order->details->post_type ) {
                            $first_name = isset( $order->details->tc_cart_info[ 'buyer_data' ][ 'first_name_post_meta' ] ) ? $order->details->tc_cart_info[ 'buyer_data' ][ 'first_name_post_meta' ] : '';
                            $last_name = isset( $order->details->tc_cart_info[ 'buyer_data' ][ 'last_name_post_meta' ] ) ? $order->details->tc_cart_info[ 'buyer_data' ][ 'last_name_post_meta' ] : '';
                            $owner_email = isset( $order->details->tc_cart_info[ 'buyer_data' ][ 'email_post_meta' ] ) ? $order->details->tc_cart_info[ 'buyer_data' ][ 'email_post_meta' ] : '';
                            $attendee_name = $first_name . ' ' . $last_name;

                        } elseif ( 'shop_order' == $order->details->post_type ) {
                            $order = wc_get_order( $order_id );
                            $first_name = $order->get_billing_first_name();
                            $last_name = $order->get_billing_last_name();
                            $owner_email = $order->get_billing_email();
                            $attendee_name = $first_name . ' ' . $last_name;
                        }
                    }

                    if ( trim( $attendee_name ) ) {
                        $attendee_data[] = [
                            'attendee_name' => $attendee_name,
                            'owner_email' => $owner_email
                        ];
                    }
                }
            }

            // Sort attendees' name in ascending order
            usort( $attendee_data, function( array $a, array $b) {
                $a_name = $a[ 'attendee_name' ] ? $a[ 'attendee_name' ] : '';
                $b_name = $b[ 'attendee_name' ] ? $b[ 'attendee_name' ] : '';

                if ( ! strcasecmp( $a_name, $b_name ) ) return 0;
                return ( strcasecmp( $a_name, $b_name ) < 0 ) ? - 1 : 1;
            } );
            ?>
            <div class="<?php echo sanitize_html_class( $wrapper_class ); ?>">
                <?php
                $attendees_emails = [];
                foreach ( $attendee_data as $attendee ) {
                    if ( array_search( $attendee[ 'owner_email' ], $attendees_emails ) === false ) {
                        $attendees_emails[] = $attendee[ 'owner_email' ];
                        echo '<div class="tc_attendee_name tc_attendee_column">' . get_avatar( $attendee[ 'owner_email' ], $size, false, $attendee[ 'attendee_name' ] ) . '<span class="tc_attendee_name">' . $attendee[ 'attendee_name' ] . '</span></div>';
                    }
                }
                ?>
            </div>
            <?php return ob_get_clean();
        }
    }

    $TC_Attendees_Gravatars = new TC_Attendees_Gravatars();
}
